@@ -1,8 +1,8 @@
 /********************************************************************** 
 *
-* This file is part of Cardpeek, the smartcard reader utility.
+* This file is part of Cardpeek, the smart card reader utility.
 *
-* Copyright 2009-2011 by 'L1L1'
+* Copyright 2009-2013 by Alain Pannetrat <L1L1@gmx.com>
 *
 * Cardpeek is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include <ctype.h>
 #endif
 #include "misc.h"
+#include "a_string.h"
 
 int bytestring_init(bytestring_t *bs, unsigned element_width)
 {
@@ -193,7 +194,7 @@ static int bs_convert_8_to_4(bytestring_t *bs,
   for (u=0;u<src->len;u++)
   {
     bs->data[u*2]=src->data[u]>>4;
-    bs->data[u*2+1]=src->data[u]&&0xF;
+    bs->data[u*2+1]=src->data[u]&0xF;
   }
   return BYTESTRING_OK;
 }
@@ -433,8 +434,8 @@ int bytestring_get_element(unsigned char* element,
 			   int pos)
 {
   if (pos<0)
-    pos=bs->len+pos;
-  if (pos<bs->len && pos>=0)
+    pos=(int)bs->len+pos;
+  if (pos<(int)bs->len && pos>=0)
   {
     *element=bs->data[pos];
     return BYTESTRING_OK;
@@ -447,8 +448,8 @@ int bytestring_set_element(const bytestring_t *bs,
 			   int pos, unsigned char element)
 {
   if (pos<0)
-    pos=bs->len+pos;
-  if (pos<bs->len && pos>=0)
+    pos=(int)bs->len+pos;
+  if (pos<(int)bs->len && pos>=0)
   {
     bs->data[pos]=element&bs->mask;
     return BYTESTRING_OK;
@@ -688,7 +689,7 @@ static void x_bytestring_append_as_digits(a_string_t* dest, const bytestring_t *
   }
 }
 
-static int x_bytestring_set(bytestring_t *bs, int bs_index, unsigned char v)
+static int x_bytestring_set(bytestring_t *bs, unsigned bs_index, unsigned char v)
 {
   if (bs_index>=bs->len) 
     bytestring_resize(bs,bs_index+1);
@@ -696,7 +697,7 @@ static int x_bytestring_set(bytestring_t *bs, int bs_index, unsigned char v)
   
   return BYTESTRING_OK;
 }
-static unsigned x_bytestring_get(bytestring_t *bs, int bs_index)
+static unsigned x_bytestring_get(bytestring_t *bs, unsigned bs_index)
 {
   if (bs_index>=bs->len)
     return 0;
@@ -741,28 +742,12 @@ static int x_bytestring_decimal_mul_add(bytestring_t *bs, unsigned mul_v, unsign
 
 static void x_bytestring_append_as_integer(a_string_t *dest, const bytestring_t *bs)
 {
-/*  bytestring_t *src; */
   bytestring_t *b10;
-  int i;
+  unsigned i;
 
   if (bs->len==0)
     return;
-/* 
-  src = bytestring_new(8);
-  b10 = bytestring_new(8);
   
-  bytestring_convert(src,bs);
-  bytestring_pushback(b10,0);
-  
-  for (i=0;i<src->len;i++)
-    x_bytestring_decimal_mul_add(b10,256,src->data[i]);
-  
-  for (i=0;i<b10->len;i++)
-    a_strpushback(dest,b10->data[b10->len-1-i]+'0');
-  
-  bytestring_free(src);
-  bytestring_free(b10);
-*/
   b10 = bytestring_new(8);
   bytestring_pushback(b10,0);
 
@@ -802,6 +787,8 @@ char *bytestring_to_format(const char *format, const bytestring_t *bs)
 		  break;
 	case 'P': x_bytestring_append_as_printable(s,bs);
 		  break;
+	case 'C': a_strncat(s,bs->len,(char *)bs->data);
+		  break;
 	case 'l': sprintf(tmp,"%i",bs->len);
 		  a_strcat(s,tmp);
 		  break;
@@ -819,7 +806,7 @@ end_this_function:
 
 double bytestring_to_number(const bytestring_t *bs)
 {
-  int i;
+  unsigned i;
   unsigned coef = 0;
   double res = 0;
 
