@@ -1,7 +1,7 @@
 --
 -- This file is part of Cardpeek, the smartcard reader utility.
 --
--- Copyright 2009-2012 by 'L1L1'
+-- Copyright 2009-2013 by 'L1L1'
 --
 -- Cardpeek is free software: you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -16,11 +16,16 @@
 -- You should have received a copy of the GNU General Public License
 -- along with Cardpeek.  If not, see <http://www.gnu.org/licenses/>.
 --
+-- @name EMV
+-- @description Bank cards 'PIN and chip'
+-- @targets 0.8
+--
 -- History:
 -- Aug 07 2010: Corrected bug in GPO command.
 -- Aug 08 2010: Corrected bug in AFL data processing
 -- Mar 27 2011: Added CVM patch from Adam Laurie.
 -- Jan 23 2012: Added UK Post Office Card Account in AID list from Tyson Key.
+-- Mar 25 2012: Added a few AIDs
 
 require('lib.tlv')
 require('lib.strict')
@@ -60,8 +65,8 @@ function ui_parse_transaction_type(node,cc)
 	if tt == nil then 
 	   return tostring(cc)
 	end
-	ui.tree_set_value(node,cc)
-	ui.tree_set_alt_value(node,tt)
+	nodes.set_attribute(node,"val",cc)
+	nodes.set_attribute(node,"alt",tt)
 	return true
 end
 
@@ -74,17 +79,24 @@ function ui_parse_CVM(node,data)
 	local subnode
 	local out 
 
-	subnode = ui.tree_add_node(node,"item","X",nil,4)
-	ui.tree_set_value(subnode,bytes.sub(data,0,3))
+	nodes.append(node,{	classname="item", 
+			      	label="X", 
+				size=4,
+				val=bytes.sub(data,0,3)})
 
-	subnode = ui.tree_add_node(node,"item","Y",nil,4)
-	ui.tree_set_value(subnode,bytes.sub(data,4,7))
+	nodes.append(node,{	classname="item", 
+				label="Y", 
+				size=4,
+				val=bytes.sub(data,4,7)})
 
 	for i= 4,(#data/2)-1 do
-		subnode = ui.tree_add_node(node,"item","CVM",i-3,2)
-		ui.tree_set_value(subnode,bytes.sub(data,i*2,i*2+1))
-		left = data[i*2]
-		right = data[i*2+1]
+		subnode = nodes.append(node, { classname="item",
+						   label="CVM",
+						   id=i-3,
+						   size=2,
+						   val=bytes.sub(data,i*2,i*2+1)})
+		left = data:get(i*2)
+		right = data:get(i*2+1)
 
 		if bit.AND(left,0x40) == 0x40 then
 			out = "Apply succeeding CV rule if this rule is unsuccessful: "
@@ -103,9 +115,9 @@ function ui_parse_CVM(node,data)
 			rightstring = CVM_REFERENCE_BYTE2[right]
 		end
 		out = out .. leftstring .. " - " .. rightstring
-		ui.tree_set_alt_value(subnode,out)
+		nodes.set_attribute(subnode,"alt",out)
 	end
-        ui.tree_set_value(node,data)
+        nodes.set_attribute(node,"val",data)
 	return true
 end
 
@@ -216,21 +228,30 @@ EMV_REFERENCE = {
 }
 
 function emv_parse(cardenv,tlv)
-	tlv_parse(cardenv,tlv,EMV_REFERENCE)
+	return tlv_parse(cardenv,tlv,EMV_REFERENCE)
 end
 
 PPSE = "#325041592E5359532E4444463031"
 PSE  = "#315041592E5359532E4444463031"
 
+-- AID_LIST enhanced with information from Wikipedia
+-- http://en.wikipedia.org/wiki/EMV
+
 AID_LIST = { 
-  "#A0000000421010",
-  "#A0000000422010",
-  "#A0000000031010",
-  "#A0000000032010",
-  "#A0000000041010",
-  "#A0000000042010",
+  "#A0000000421010", -- French CB
+  "#A0000000422010", -- French CB
+  "#A0000000031010", -- Visa credit or debit
+  "#A0000000032010", -- Visa electron
+  "#A0000000032020", -- V pay
+  "#A0000000032010", -- Visa electron
+  "#A0000000041010", -- Mastercard credit or debit 
+  "#A0000000042010", -- 
+  "#A0000000043060", -- Mastercard Maestro
+  "#A0000000046000", -- Mastercard Cirrus 
   "#A00000006900",   -- FR Moneo
   "#A0000001850002", -- UK Post Office Card Account card
+  "#A00000002501",   -- American Express
+  "#A0000001523010", -- Diners club
 }
 
 EXTRA_DATA = { 0x9F36, 0x9F13, 0x9F17, 0x9F4D, 0x9F4F }
@@ -240,7 +261,6 @@ function emv_process_ppse(cardenv)
 	local sw, resp
 	local APP
 	local dirent
-	local i
 
 	sw, resp = card.select(PPSE)
 
@@ -250,15 +270,13 @@ function emv_process_ppse(cardenv)
 	end
 
 	-- Construct tree
-	APP = ui.tree_add_node(cardenv,"application","application",PPSE)
+	APP = nodes.append(cardenv, {classname="application", label="application", id=PPSE})
 	emv_parse(APP,resp)
 
 	AID_LIST = {}
 
-	dirent = ui.tree_find_all_nodes(APP,nil,"4F")
-
-	for i=1,#dirent do
-	        aid = tostring(ui.tree_get_value(dirent[i]))
+	for dirent in  nodes.find(APP,{ id="4F" }) do
+	        aid = tostring(nodes.get_attribute(dirent,"val"))
 		log.print(log.INFO,"PPSE contains application #" .. aid)
 	        table.insert(AID_LIST,"#"..aid)
 	end
@@ -277,16 +295,15 @@ function emv_process_pse(cardenv)
 	local RECORD
 	local aid
 	local warm_atr
+	local tag4F
 
         sw, resp = card.select(PSE)
 	
 	-- Could it be a french card?
 	if sw == 0x6E00 then 
-	   local ATR
 	   card.warm_reset()
 	   warm_atr = card.last_atr()
-	   ATR = ui.tree_add_node(cardenv, "block", "ATR", "warm", #warm_atr)
-	   ui.tree_set_value(ATR,warm_atr)
+	   nodes.append(cardenv, {classname="atr", label="ATR", id="warm", size=#warm_atr, val=warm_atr})
            sw, resp = card.select(PSE)
 	end
 
@@ -301,25 +318,28 @@ function emv_process_pse(cardenv)
 	end
 
 	-- Construct tree
-	APP = ui.tree_add_node(cardenv,"application","application",PSE)
+	APP = nodes.append(cardenv, {classname="application", label="application", id=PSE})
 	emv_parse(APP,resp)
 
-	ref = ui.tree_find_node(APP,nil,"88")
+	ref = nodes.find_first(APP,{id="88"})
 	if (ref) then
-	   sfi = ui.tree_get_value(ref)
-	   FILE = ui.tree_add_node(APP,"file","file",sfi[0])
+	   sfi = nodes.get_attribute(ref,"val")
+	   FILE = nodes.append(APP,{classname="file", label="file", id=sfi[0]})
 
 	   rec = 1
 	   AID_LIST = {}
 	   repeat 
-	     sw,resp = card.read_record(sfi[0],rec)
+	     sw,resp = card.read_record(sfi:get(0),rec)
 	     if sw == 0x9000 then
-	        RECORD = ui.tree_add_node(FILE,"record","record",tostring(rec))
+	        RECORD = nodes.append(FILE, {classname="record", label="record", id=tostring(rec)})
 	        emv_parse(RECORD,resp)
-	        aid = tostring(ui.tree_get_value(ui.tree_find_node(RECORD,nil,"4F")))
-		log.print(log.INFO,"PSE contains application #" .. aid)
-	        table.insert(AID_LIST,"#"..aid)
-		rec = rec + 1
+		tag4F = nodes.find_first(RECORD,{id="4F"})
+		if (tag4F) then
+	           aid = tostring(nodes.get_attribute(tag4F,"val"))
+		   log.print(log.INFO,"PSE contains application #" .. aid)
+	           table.insert(AID_LIST,"#"..aid)
+		   rec = rec + 1
+		end
              end
 	   until sw ~= 0x9000
 	else
@@ -333,11 +353,14 @@ function visa_process_application_logs(application, log_sfi, log_max)
 	local LOG_FILE
 	local LOG_DATA
 	local LOG_FORMAT
-	local REC
 	local log_recno
 
-	LOG_FILE   = ui.tree_add_node(application,"file","file",tostring(log_sfi),nil)
-	LOG_DATA   = ui.tree_add_node(LOG_FILE,"item","log data")
+	LOG_FILE   = nodes.append(application,{ classname="file", 
+						    label="file",
+						    id=tostring(log_sfi)})
+
+	LOG_DATA   = nodes.append(LOG_FILE, { classname="item", 
+						  label = "log data"})
  	
 	log.print(log.INFO,string.format("Reading LOG SFI %i",log_sfi))
 	for log_recno =1,log_max do
@@ -346,8 +369,11 @@ function visa_process_application_logs(application, log_sfi, log_max)
                   log.print(log.WARNING,"Read log record failed")
 		  break
 	   else
-	          REC = ui.tree_add_node(LOG_DATA,"record","record",log_recno,#resp)
-		  ui.tree_set_value(REC,resp)
+	          nodes.append(LOG_DATA,{ classname = "record", 
+					      label="record",
+					      id=log_recno,
+					      size=#resp,
+					      val=resp})
 	   end
 	end 
 
@@ -376,21 +402,14 @@ function emv_process_application_logs(application, log_sfi, log_max)
 	local data
 	local currency_code, currency_name, currency_digits
 
-	LOG_FILE   = ui.tree_add_node(application,"file","file",log_sfi)
-	LOG_FORMAT = ui.tree_add_node(LOG_FILE,"item","log format")
-	LOG_DATA   = ui.tree_add_node(LOG_FILE,"item","log data")
+	LOG_FILE   = nodes.append(application,     { classname="file", label="log file", id=log_sfi})
+	LOG_FORMAT = nodes.append(LOG_FILE,	   { classname="block", label="log format"})
  	
-	sw, log_format = card.get_data(0x9F4F)
-	if sw ~=0x9000 then
-	   log.print(log.ERROR,"Failed to get log format information, transaction logs won't be analyzed")
-	   return false
-	end
-
-	log_tag, log_format = asn1.split(log_format)
+	log_format = application:find_first({id = "9F4F"}):get_attribute("val")
 
 	i = 1
 	item = ""
-	ui.tree_set_value(LOG_FORMAT,log_format);
+	nodes.set_attribute(LOG_FORMAT,"val",log_format);
 	while log_format 
 	do
 	    tag, log_format = asn1.split_tag(log_format)
@@ -400,7 +419,7 @@ function emv_process_application_logs(application, log_sfi, log_max)
 	    i = i+1
 	    item = item .. string.format("%X[%d] ", tag, len);
 	end
-	ui.tree_set_alt_value(LOG_FORMAT,item);
+	nodes.set_attribute(LOG_FORMAT,"alt",item);
 
 	log.print(log.INFO,string.format("Reading LOG SFI %i",log_sfi))
 	for log_recno =1,log_max do
@@ -409,19 +428,23 @@ function emv_process_application_logs(application, log_sfi, log_max)
                   log.print(log.WARNING,"Read log record failed")
 		  break
 	   else
-	          REC = ui.tree_add_node(LOG_DATA,"record","record",log_recno,#resp)
+	          REC = nodes.append(LOG_FILE,{classname="record", label="record", id=log_recno, size=#resp})
 		  item_pos = 0
 		  ITEM_AMOUNT = nil
 		  currency_digits = 0
 		  for i=1,#log_items do
 		     if log_items[i][3] then
-		        ITEM = ui.tree_add_node(REC,"item",log_items[i][3],nil,log_items[i][2])
+		        ITEM = nodes.append(REC,{ classname="item", 
+						      label=log_items[i][3], 
+						      size=log_items[i][2] })
 		     else
-		        ITEM = ui.tree_add_node(REC,"item",string.format("tag %X",log_items[i][1]),nil,log_items[i][2])
+		        ITEM = nodes.append(REC,{ classname="item", 
+						      label=string.format("tag %X",log_items[i][1]),
+						      size=log_items[i][2] })
 		     end
 
 		     data = bytes.sub(resp,item_pos,item_pos+log_items[i][2]-1)
-	             ui.tree_set_value(ITEM,data)
+	             nodes.set_attribute(ITEM,"val",data)
 
 		     if log_items[i][1]==0x9F02 then
 		        ITEM_AMOUNT=ITEM
@@ -429,7 +452,7 @@ function emv_process_application_logs(application, log_sfi, log_max)
 	                currency_code   = tonumber(tostring(data))
 		        currency_name   = iso_currency_code_name(currency_code)
 	                currency_digits = iso_currency_code_digits(currency_code)
-	                ui.tree_set_alt_value(ITEM,currency_name)
+	                nodes.set_attribute(ITEM,"alt",currency_name)
 		     elseif log_items[i][4] then
 		        log_items[i][4](ITEM,data)
                      end
@@ -437,7 +460,8 @@ function emv_process_application_logs(application, log_sfi, log_max)
 	          end
 
 	          if ITEM_AMOUNT then
-		     ui.tree_set_alt_value(ITEM_AMOUNT,string.format("%.2f",tostring(ui.tree_get_value(ITEM_AMOUNT))/(10^currency_digits)))
+		     local amount = tostring(nodes.get_attribute(ITEM_AMOUNT,"val"))
+		     nodes.set_attribute(ITEM_AMOUNT,"alt",string.format("%.2f",amount/(10^currency_digits)))
 		  end
 	   end
 	end 
@@ -460,37 +484,70 @@ function emv_process_application(cardenv,aid)
 	   return false
 	end
 
-
 	-- Process 'File Control Infomation' and get PDOL
 	local APP
+	local FCI
 	
-	APP = ui.tree_add_node(cardenv,"application","application",aid)
-	emv_parse(APP,resp)
-	ref = ui.tree_find_node(APP,nil,"9F38")
+	APP = nodes.append(cardenv, { classname = "application", label = "application", id=aid })
+	FCI = nodes.append(APP, { classname = "header", label = "answer to select", size=#resp, val=resp })
+	emv_parse(FCI,resp)
+	ref = nodes.find_first(FCI,{id="9F38"})
 	if (ref) then
-	   pdol = ui.tree_get_value(ref)
+	   pdol = nodes.get_attribute(ref,"val")
 	else
 	   pdol = nil;
 	end
 
+	-- INITIATE get processing options, now that we have pdol
+	local GPO
 
-	-- find LOG INFO
+        if ui.question("Issue a GET PROCESSING OPTIONS command?",{"Yes","No"})==1 then
+            -- Get processing options
+            log.print(log.INFO,"Attempting GPO")
+            sw,resp = card.get_processing_options(pdol)
+            if sw ~=0x9000 then
+                if pdol then
+                    -- try empty GPO just in case the card is blocking some stuff
+                    log.print(log.WARNING,
+                              string.format("GPO with data failed with code %X, retrying GPO without data",sw))
+                    sw,resp = card.get_processing_options(nil)	
+                end
+           end
+           if sw ~=0x9000 then
+                log.print(log.ERROR,"GPO Failed")
+		ui.question("GET PROCESSING OPTIONS failed, the script will continue to read the card",{"OK"})
+           else
+ 	   	GPO = nodes.append(APP,{classname="block",label="processing_options", size=#resp, val=resp})
+		emv_parse(GPO,resp)
+	   end
+	end
+
+	-- Read extra data 
+	--extra = nodes.append(APP,{classname="block",label="extra emv data"})
+	for j=1,#EXTRA_DATA do
+	    sw,resp = card.get_data(EXTRA_DATA[j])
+	    if sw == 0x9000 then
+	       emv_parse(APP,resp):set_attribute("classname","block")
+	    end
+	end
+	
+	-- find LOG INFO 
 	local LOG
 	local logformat
 	
 	logformat=nil
-	ref = ui.tree_find_node(APP,nil,"9F4D")
+	ref = nodes.find_first(APP,{id="9F4D"})
 	-- I've seen this on some cards :
 	if ref==nil then
 	   -- proprietary style ?
-	   ref = ui.tree_find_node(APP,nil,"DF60")
+	   ref = nodes.find_first(APP,{id="DF60"})
 	   logformat = "VISA"
 	else
 	   logformat = "EMV"
 	end
 
 	if ref then
-           LOG = ui.tree_get_value(ref)
+           LOG = nodes.get_attribute(ref,"val")
 	else
 	   sw, resp = card.get_data(0x9F4D)
       	   if sw==0x9000 then
@@ -515,40 +572,6 @@ function emv_process_application(cardenv,aid)
 	end
 
 
-	-- INITIATE get processing options
-	local GPO
-
-        if ui.question("Issue a GET PROCESSING OPTIONS command?",{"Yes","No"})==1 then
-            -- Get processing options
-            log.print(log.INFO,"Attempting GPO")
-            sw,resp = card.get_processing_options(pdol)
-            if sw ~=0x9000 then
-                if pdol then
-                    -- try empty GPO just in case the card is blocking some stuff
-                    log.print(log.WARNING,
-                              string.format("GPO with data failed with code %X, retrying GPO without data",sw))
-                    sw,resp = card.get_processing_options(nil)	
-                end
-           end
-           if sw ~=0x9000 then
-                log.print(log.ERROR,"GPO Failed")
-		ui.question("GET PROCESSING OPTIONS failed, the script will continue to read the card",{"OK"})
-           else
- 	   	GPO = ui.tree_add_node(APP,"item","processing_options");
-	    	emv_parse(GPO,resp)
-	   end
-	end
-
-
-	-- find AFL
-	--ref = ui.tree_find_node(GPO,nil,"80")
-	--if (ref) then
-	--	AFL = bytes.sub(ui.tree_get_value(ref),2)
-	--else
-	--	ref = ui.tree_find_node(GPO,nil,"94")
-	--	AFL = ui.tree_get_value(ref)
-	--end
-
 	local sfi_index
 	local rec_index
 	local SFI
@@ -557,7 +580,7 @@ function emv_process_application(cardenv,aid)
         for sfi_index=1,31 do
 		SFI = nil
 
-		if (logformat==nil or LOG[0]~=sfi_index) then
+		if (logformat==nil or LOG:get(0)~=sfi_index) then
 
 	    		for rec_index=1,255 do
 				log.print(log.INFO,string.format("Reading SFI %i, record %i",sfi_index, rec_index))
@@ -567,10 +590,13 @@ function emv_process_application(cardenv,aid)
 				    break
 			        else
 				    if (SFI==nil) then
-	   		    		    SFI = ui.tree_add_node(APP,"file","file",sfi_index)
+	   		    		    SFI = nodes.append(APP,{classname="file", label="file",	id=sfi_index})
+				    end
+			            REC = nodes.append(SFI,{classname="record", label="record", id=rec_index})
+			            if (emv_parse(REC,resp)==false) then
+					    nodes.set_attribute(REC,"val",resp)
+					    nodes.set_attribute(REC,"size",#resp)
 				    end
-			            REC = ui.tree_add_node(SFI,"record","record",rec_index)
-			            emv_parse(REC,resp)
 		        	end
 	
 		        end -- for rec_index
@@ -581,20 +607,12 @@ function emv_process_application(cardenv,aid)
 
 	-- Read logs if they exist
 	if logformat=="EMV" then
-           emv_process_application_logs(APP,LOG[0],LOG[1])
+           emv_process_application_logs(APP,LOG:get(0),LOG:get(1))
 	elseif logformat=="VISA" then
-	   visa_process_application_logs(APP,LOG[0],LOG[1])
+	   visa_process_application_logs(APP,LOG:get(0),LOG:get(1))
 	end
 
-	-- Read extra data 
-	extra = ui.tree_add_node(APP,"block","extra emv data")
-	for j=1,#EXTRA_DATA do
-	    sw,resp = card.get_data(EXTRA_DATA[j])
-	    if sw == 0x9000 then
-	       emv_parse(extra,resp)
-	    end
-	end
-	
+
 	return true
 end
 
@@ -635,15 +653,12 @@ function emv_process_cplc(cardenv)
 	sw,resp = card.get_data(0x9F7F)
 	if sw == 0x9000 then
 	   cplc_tag, cplc_data = asn1.split(resp)
-	   CPLC      = ui.tree_add_node(cardenv,"block","cpcl data","9F7F",#cplc_data)
-	   ui.tree_set_value(CPLC,cplc_data) 
+	   CPLC      = nodes.append(cardenv,{classname="block", label="cpcl data", id="9F7F", size=#cplc_data})
+	   nodes.set_attribute(CPLC,"val",cplc_data) 
 	   pos = 0
 	   for i=1,#CPLC_DATA do
-	       ref2 = ui.tree_add_node(CPLC,
-				       "item",
-				       CPLC_DATA[i][1],
-				       pos);
-	       ui.tree_set_value(ref2,bytes.sub(cplc_data,pos,pos+CPLC_DATA[i][2]-1))
+	       ref2 = nodes.append(CPLC,{classname="item", label=CPLC_DATA[i][1], id=pos});
+	       nodes.set_attribute(ref2,"val",bytes.sub(cplc_data,pos,pos+CPLC_DATA[i][2]-1))
 	       pos = pos + CPLC_DATA[i][2]
 	   end
 	end
