@@ -1,8 +1,8 @@
 /********************************************************************** 
 *
-* This file is part of Cardpeek, the smartcard reader utility.
+* This file is part of Cardpeek, the smart card reader utility.
 *
-* Copyright 2009 by 'L1L1'
+* Copyright 2009-2013 by Alain Pannetrat <L1L1@gmx.com>
 *
 * Cardpeek is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -22,21 +22,15 @@
 #ifndef GUI_H
 #define GUI_H
 
-#include "cardtree.h"
-#include "bytestring.h"
-
-extern cardtree_t* CARDTREE;
-
-typedef void (*application_callback_t)(const char *);
-
 int gui_init(int *argc, char ***argv);
 
-int gui_create(application_callback_t run_script_cb,
-	       application_callback_t run_command_cb);
+int gui_create(void);
 
 int gui_run(void);
 
-void gui_update(int lag_allowed);
+void gui_exit(void);
+
+void gui_update(unsigned lag_allowed);
 
 void gui_expand_view(void);
 
@@ -53,12 +47,6 @@ char** gui_select_file(const char *title,
    		       const char *path,
    		       const char *filename);
 
-void gui_about(void);
-
-void gui_reader_print_data(unsigned event,
-			   const bytestring_t *command,
-			   unsigned short sw,
-			   const bytestring_t *response,
-			   void *extra_data);
+void gui_set_title(const char *title);
 
 #endif
