@@ -1,8 +1,8 @@
 /********************************************************************** 
 *
-* This file is part of Cardpeek, the smartcard reader utility.
+* This file is part of Cardpeek, the smart card reader utility.
 *
-* Copyright 2009 by 'L1L1'
+* Copyright 2009-2013 by Alain Pannetrat <L1L1@gmx.com>
 *
 * Cardpeek is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -23,22 +23,24 @@
 #define PATH_CONFIG_H
 
 enum {
-  CONFIG_FOLDER_CARDTREES,
-  CONFIG_FOLDER_SCRIPTS_OTHER,
-  CONFIG_FOLDER_HOME,
-  CONFIG_FOLDER_CARDPEEK,
-  CONFIG_FOLDER_SCRIPTS,
-  CONFIG_FOLDER_LOGS,
-  CONFIG_FILE_CONFIG,
-  CONFIG_FILE_LOG,
-  CONFIG_FILE_SCRIPT_VERSION,
-  NUM_CONFIG_OPTIONS
+  PATH_CONFIG_FOLDER_WORKING,
+  PATH_CONFIG_FILE_CARDPEEK_LOG,
+  PATH_CONFIG_FOLDER_CARDPEEK,
+  PATH_CONFIG_FOLDER_SCRIPTS,
+  PATH_CONFIG_FOLDER_REPLAY,
+  PATH_CONFIG_FOLDER_OLD_REPLAY,
+  PATH_CONFIG_FILE_CONFIG_LUA,
+  PATH_CONFIG_FILE_CARDPEEK_RC,
+  PATH_CONFIG_FILE_VERSION,
+  PATH_CONFIG_FILE_SMARTCARD_LIST_TXT,
+  PATH_CONFIG_FILE_SMARTCARD_LIST_DOWNLOAD,
+  NUM_PATH_CONFIG_OPTIONS
 };
 
 
-int config_init(void);
-const char *config_get_string(unsigned index);
-int config_set_string(unsigned index, const char *path);
-void config_release(void);
+int path_config_init(void);
+const char *path_config_get_string(unsigned index);
+int path_config_set_string(unsigned index, const char *path);
+void path_config_release(void);
 
 #endif
